/**
 * Namespace qui regroupe toutes les fonctions en rapport avec le LDAP - API et fonctions internes
 * @namespace LDAP
 */

/**
 * @file Importe la configuration du LDAP au sein de l'application, et remplace certaines valeurs en fonction des variables d'environnement.
 * @memberof LDAP
 * @author manifold, hawkspar
 */

import fs from 'fs';
import path from 'path';
import colors from 'colors';
import dotenv from 'dotenv';
dotenv.config();

// Point central ; tous les champs de la BDD sont 'cachés' dans config.json et pas visibles directement
let path_config = path.resolve(__dirname, '..', '..', '..', 'ldap_config.json');
console.log(colors.cyan("Loading LDAP config file from "+path_config));
export const ldapConfig = JSON.parse(fs.readFileSync(path_config).toString());

// Override config server from environment
if (process.env.LDAP_URI != null) {
    ldapConfig.server = process.env.LDAP_URI;
}
else {
    if (process.env.TARGET_ENV == `production`)     { ldapConfig.server = ldapConfig.server_prod; }
    else                                            { ldapConfig.server = ldapConfig.server_dev; }
}

// Gestion des super-identifiants
let path_credentials = path.resolve(__dirname, '..', '..', '..', 'ldap_credentials.json')
console.log(colors.cyan("Loading LDAP credentials from "+path_credentials));
export const credentialsLdapConfig = JSON.parse(fs.readFileSync(path_credentials).toString());

// Data formats and useful constants
export const categories = ["admins","speakers","members","followers"];

/**
 * @interface userData
 * @desc Interface avec toutes les données extractables pour un utilisateur.
 * @var {string?} uid - Identifiant utilisateur
 * TBA @var {string[]?} admins - Liste des gid (group id, inclus section sportive, binet, PA...) dont l'utilisateur est admin ; pas forcément sous-liste de groups
 * TBA @var {string[]?} speakers - Liste des gid dont l'utilisateur est porte-parole ; pas forcément sous-liste de groups
 * @var {string[]?} members - Liste des gid dont l'utilisateur est membre
 * TBA @var {string[]?} followers - Liste des gid dont l'utilisateur est sympathisant
 * @var {string?} givenName - Prénom
 * @var {string?} lastName - Nom
 * @var {string?} nickname - Surnom
 * @var {string?} photo - Bytestring de la photo de l'utilisateur
 * @var {string?} birthdate - Date d'anniversaire
 * TBA @var {string?} nationality - Nationalité d'origine
 * @var {string?} promotion - Année(s) de promo
 * @var {string?} phone - Numéro(s) de téléphone
 * @var {string[]} address - Adresse(s)
 * @var {string[]?} mail - Adresse(s) courriel
 * @var {string?} password - Mot de passe généré en amont (utilisé seulement à l'initialialisation, pas stocké bien sûr)
 * @var {string[]?} ips - Adresse(s) ip
 * @var {string?} directory - Adresse soft des données utilisateurs
 * @var {string?} login - Astuce de root flemmard
 * @arg {string?} readPerm - Permissions spéciales BR
 * @var {string?} writePerm - Permissions spéciales BR
 * @var {string[]?} forlifes - Alias BR (attention le filtre .fkz n'est plus fonctionnel)
 * @memberof LDAP
 */
export class userData {
    uid?: string;
    admins?: string[];
    speakers?: string[];
    members?: string[];
    followers?: string[];
    password?: string;
    givenName?: string;
    lastName?: string;
    nickname?: string;
    promotion?: string;
    photo?: string;
    birthdate?: string;
    nationality?: string;
    phone?: string;
    address?: string;
    mail?: string;
    ips?: string[];
    directory?: string;
    login?: string;
    readPerm?: string;
    writePerm?: string;
    forlifes?: string[];
    sport?: string;
}

/**
 * @interface groupData
 * @var {string} gid - Identifiant du groupe
 * @var {string} name - Nom du groupe (souvent son nom mais pas nécessairement)
 * @var {string} type - Statut du groupe ; binet, section sportive... (actuellement juste 'binet' ou 'free')
 * @var {string[]} members - Liste des membres du groupe
 * @var {string[]} admins - Liste des admins du groupe ; supposée être une sous-liste de la précédente
 * @var {string} description - Description du groupe (facultatif)
 * @var {string[]} childs - Liste des groupes enfants de première génération de celui-ci (les admins du groupe seront admins de ce groupe et des enfants suivants)
 * @var {string[]} parents - Liste des groupes directement parents de celui-ci (les membres du groupe seront membres de ce groupe et des parents suivants) ; symétrique du précédent
 * @memberof LDAP
 */
export class groupData {
    gid: string;
	name: string;
	type: string;
    members: string[];
    speakers: string[];
    followers: string[];
    admins: string[];
    description?: string;
    childs?: string[];
    parents?: string[];
}
